package arzumify.networked.api;

import arzumify.networked.impl.BasePresence;
import arzumify.networked.impl.MatchMaker;
import arzumify.presence.maths.Vec2i;
import net.fabricmc.fabric.api.event.lifecycle.v1.ServerChunkEvents;
import net.fabricmc.fabric.api.event.lifecycle.v1.ServerTickEvents;
import net.minecraft.server.world.ServerWorld;
import net.minecraft.util.math.ChunkPos;
import net.minecraft.world.chunk.WorldChunk;

import java.util.HashSet;

abstract public class Network extends BasePresence implements ServerTickEvents.EndWorldTick, ServerChunkEvents.Unload, ServerChunkEvents.Load {
    private final HashSet<Node> nodes = new HashSet<>();
    private final NetworkFactory factory;
    private static final HashSet<ChunkPos> unloadedChunks = new HashSet<>();

    public Network(NetworkFactory parentFactory) {
        super((short) 1);
        this.factory = parentFactory;
    }

    public void Merge(Network other) {
        MatchMaker.matchMaker.Remove(other, MatchMaker.networkDiscovery);
        super.presence.addNewBasePosition(other.presence.basePositions);
        nodes.addAll(other.nodes);
    }

    public void Add(Node node) {
        super.presence.addNewBasePosition(node.presence.basePositions);
        nodes.add(node);
    }

    public void Remove(Node node) {
        System.out.println("Removing");
        System.out.println(nodes);
        nodes.remove(node);
        MatchMaker.matchMaker.Remove(this, MatchMaker.networkDiscovery);
        rebuildNetwork();
    }

    private void rebuildNetwork() {
        for (Node splitNode : nodes) {
            var found = MatchMaker.matchMaker.Search(splitNode, MatchMaker.networkDiscovery);
            System.out.println("Removing node lol");
            System.out.println(found);
            if (found != null && !found.isEmpty()) {
                Network previous = null;
                for (var basePresence : found) {
                    if (basePresence instanceof Network network) {
                        if (previous == null) {
                            network.Add(splitNode);
                        } else {
                            network.Merge(previous);
                        }
                        previous = network;
                    }
                }
            } else {
                System.out.println("making a new net");
                var network = factory.create();
                network.Add(splitNode);
                System.out.println("adding the net");
                MatchMaker.matchMaker.Add(network, MatchMaker.networkDiscovery);
            }
        }
    }

    @Override
    public void onEndTick(ServerWorld serverWorld) {
        tick(serverWorld);
    }

    /**
     * The tick method is called on every tick.
     *
     * @param serverWorld The world that the tick was run in
     */
    abstract public void tick(ServerWorld serverWorld);

    @Override
    public void onChunkUnload(ServerWorld serverWorld, WorldChunk worldChunk) {
        var chunk = worldChunk.getPos();
        var needsRebuildFactory = false;
        for (var loadedChunk : presence.baseChunkPresence) {
            if (new Vec2i(chunk.x, chunk.z).equals(loadedChunk)) {
                unloadedChunks.add(chunk);
                needsRebuildFactory = true;
            }
        }
        if (needsRebuildFactory) {
            rebuildNetwork();
        }
    }

    @Override
    public void onChunkLoad(ServerWorld serverWorld, WorldChunk worldChunk) {
        if (unloadedChunks.contains(worldChunk.getPos())) {
            rebuildNetwork();
        }
    }
}
