package arzumify.networked.api;

import arzumify.networked.impl.MatchMaker;
import arzumify.presence.maths.Vec3i;
import net.minecraft.block.Block;
import net.minecraft.block.BlockState;
import net.minecraft.entity.LivingEntity;
import net.minecraft.item.ItemStack;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;

import java.util.HashMap;

public class NetworkedBlock extends Block {
    public NetworkedBlock(Settings settings, NetworkFactory factory) {
        super(settings);
        this.factory = factory;
    }

    private final NetworkFactory factory;
    private final HashMap<BlockPos, Network> network = new HashMap<>();
    private final HashMap<BlockPos, Node> node = new HashMap<>();

    @Override
    @Deprecated
    @SuppressWarnings("deprecation")
    public void onStateReplaced(BlockState state, World world, BlockPos pos, BlockState newState, boolean moved) {
        if (!world.isClient) {
            onStateReplaced(pos);
        }
    }

    public void onStateReplaced(BlockPos pos) {
        network.get(pos).Remove(node.get(pos));
    }

    @Override
    public void onPlaced(World world, BlockPos pos, BlockState state, LivingEntity placer, ItemStack itemStack) {
        super.onPlaced(world, pos, state, placer, itemStack);
        if (!world.isClient) {
            onPlaced(pos);
        }
    }


    public void onPlaced(BlockPos pos) {
        var presence = new Node(new Vec3i(pos.getX(), pos.getY(), pos.getZ()));
        node.put(pos, presence);
        var found = MatchMaker.matchMaker.Search(presence, MatchMaker.networkDiscovery);
        if (!found.isEmpty()) {
            Network previous = null;
            for (var basePresence : found) {
                if (basePresence instanceof Network foundNetwork) {
                    if (previous == null) {
                        foundNetwork.Add(presence);
                        this.network.put(pos, foundNetwork);
                    } else {
                        foundNetwork.Merge(previous);
                    }
                    previous = foundNetwork;
                }
            }
        } else {
            var network = factory.create();
            network.Add(presence);
            MatchMaker.matchMaker.Add(network, MatchMaker.networkDiscovery);
        }
    }
}
